import os
from dotenv import load_dotenv
from enum import Enum

load_dotenv()

SOAP_API_URL = 'https://myapi.ppl.cz/MyApi.svc' # os.getenv('SOAP_API_URL')
SOAP_ACTION_URL = 'http://myapi.ppl.cz/v1/IMyApi2/' # os.getenv('SOAP_ACTION_URL')
REST_API_URL = 'https://api.dhl.com/ecs/ppl/myapi2/' # os.getenv('REST_API_URL')
REST_OAUTH2_TOKEN_URL = 'https://api.dhl.com/ecs/ppl/myapi2/login/getAccessToken' # os.getenv('REST_OAUTH2_TOKEN_URL')
REST_GRANT_TYPE = 'client_credentials' # os.getenv('REST_GRANT_TYPE')
# REST_CLIENT_ID = None # os.getenv('REST_CLIENT_ID')
# REST_CLIENT_SECRET = None # os.getenv('REST_CLIENT_SECRET')
# SOAP_CUST_ID = None # os.getenv('SOAP_CUST_ID')
# SOAP_PASSWORD = None # os.getenv('SOAP_PASSWORD')
# SOAP_USERNAME = None # os.getenv('SOAP_USERNAME')
# SOAP_API_URL = 'https://myapi.ppl.cz/MyApi.svc' # os.getenv('SOAP_API_URL')
SOAP_AUTH_TOKEN_MAX_AGE = 1800 # int(os.getenv('SOAP_AUTH_TOKEN_MAX_AGE'))


SOAP_HEADERS = {
    'Content-Type': 'text/xml; charset=utf-8',
    'SOAPAction': os.getenv('SOAP_ACTION_URL'),
}

class Enum2(Enum):
    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_ 

# available countries and currencies
class Country(Enum2):
    CZ = 'CZ'
    DE = 'DE'
    GB = 'GB'
    SK = 'SK'
    AT = 'AT'
    PL = 'PL'
    CH = 'CH'
    FI = 'FI'
    HU = 'HU'
    SI = 'SI'
    LV = 'LV'
    EE = 'EE'
    LT = 'LT'
    BE = 'BE'
    DK = 'DK'
    ES = 'ES'
    FR = 'FR'
    IE = 'IE'
    IT = 'IT'
    NL = 'NL'
    NO = 'NO'
    PT = 'PT'
    SE = 'SE'
    RO = 'RO'
    BG = 'BG'
    HR = 'HR'
    LU = 'LU'

class Currency(Enum2):
    CZK = 'CZK'
    EUR = 'EUR'
    PLN = 'PLN'
    HUF = 'HUF'
    RON = 'RON'

# return codes
class ReturnCode(Enum2):
    LOADED_SUCCESSFULLY = 0
    INVALID_OR_MISSING_PACKAGE_NUMBER = 1000
    UNKNOWN_PRODUCT_TYPE = 1001
    BAD_SENDER_ADDRESS = 1002
    BAD_RECIPIENT_ADDRESS = 1003
    INVALID_COD_VARIABLE_SYMBOL = 1004
    INVALID_SPECIAL_SYMBOL = 1005
    COD_PRICE_AND_COD_CURRENCY_NOT_FILLED_BOTH = 1006
    COD_NEGATIVE_PRICE = 1007
    INVALID_BANK_INFORMATIONS_FILLED_BANK_ACCOUNT_CODE_AND_ALSO_IBAN_SWIFT = 1008
    BANK_ACCOUNT_AND_BANK_CODE_MUST_BE_FILLED_BOTH = 1009
    IBAN_AND_SWIFT_MUST_BE_FILLED_BOTH = 1010
    UNKNOWN_BANK_CODE = 1011
    UNKNOWN_SWIFT_CODE = 1012
    FILLED_COD_INFORMATIONS_FOR_NONCOD_PRODUCT = 1013
    
    UNKNOWN_PARCELSHOP_CODE = 1023
    INVALID_PACKAGE_WEIGHT = 1024
    INSURANCE_PRICE_AND_CURRENCY_MUST_BE_FILLED_BOTH = 1025
    INSURANCE_NEGATIVE_PRICE = 1026
    UNKNOWN_COD_CURRENCY = 1027
    UNKNOWN_INSURANCE_CURRENCY = 1028
    UNKNOWN_PACKAGE_SERVICE = 1029
    UNKNOWN_EXTERNAL_NUMBER = 1030
    INVALID_COUNTRY_IN_SENDER_ADDRESS = 1031
    INVALID_COUNTRY_IN_RECIPIENT_ADDRESS = 1032
    INVALID_INFORMATIONS_ABOUT_WEIGHTED_PACKAGES = 1033
    UNKNOWN_ROUTES = 1034
    INVALID_PACKAGE_WEIGHT_100X = 1035 # DUPLICITY
    SATURDAY_DELIVERY_FILLED_AND_SATURDAY_ROUTE_NOT_OR_VICE_VERSA = 1036
    MISSING_INPUT_DESTINATION = 1037
    MISSING_OUTPUT_DESTINATION = 1038
    FILLED_ROUTE_TYPE_IN_AND_NOT_ROUTE_TYPE_OUT_OR_VICE_VERSA_TWO_TIMES_FILLED_ROUTE_TYPE_OUT_OR_ROUTE_TYPE_IN = 1039
    UNKNOWN_PACKAGE_FLAG = 1040


    # CreatePackages
    SPECTAKEDATE_MUST_BE_EQUAL_OR_GREATER_THAN_THE_CURRENT_DATE = 1033
    # UNKNOWN_ROUTES = 1034 <-- already defined
    SERVICE_IS_NOT_RELEVANT_FOR_PRODUCT_TYPE = 1035
    
    SD_FLAG_CANT_BE_SPECIFIED_FOR_THIS_PRODUCT_TYPE = 1043
    RECIPIENT_COUTRY_ZIPCODE_WRONG_CHECK_THE_FORMAT = 1046
    BAD_SENDER_ZIPCODE_FORMAT = 1047
    PARAMETER_SPECTAKEDATE_MUST_BE_SPECIFIED = 1048
    COD_PRICE_IS_GREATER_THAN_THE_MAXIMUM_ALLOWABLE_LIMIT_FOR_THE_COUNTRY = 1049
    MAXIMUM_NUMBER_OF_PACKAGES_EXCEEDED = 1050
    VALUE_OF_KEY_WITH_NAME_MAXINSURANCE_MAXCODPRICE_MAXPACKAGESINCOUNT_IS_NOT_DECIMAL = 1051
    THE_PACKNUMBER_IS_REQUIRED_FIELD = 1052
    DUPLICIT_PACKNUMBER = 1053
    INSURANCE_PRICE_MUST_BE_LOWER_OR_EQUAL_TO = 1054
    MAXIMUM_NUMBER_OF_PACKAGES_EXCEEDED_100X = 1055 #//!FIXME duplicit with 105
    THE_PACKNUMBER_IS_REQUIRED_FIELD_100X = 1057  #//!FIXME duplicit with 105
    DUPLICIT_PACKNUMBER_100X = 1058 #//!FIXME duplicit with 105
    PACKAGE_PACKNUMBER_IS_NOT_AUTHORIZED_TO_CREATE_BACK_DOCUMENTS = 1059
    PACKAGE_PACKNUMBER_DOCUMENTSBACK_PACKNUMBER_IS_MISSING = 1060
    PACKAGE_PACKNUMBER_SERVICE_WITH_CODE_DB_IS_MISSING = 1061
    INVALID_RECIPIENT_ADDRESS_FOR_BACK_DOCUMENTS = 1062
    RECIPIENT_ZIP_CODE_DOESNT_SUPPORT_MORNING_DELIVERY = 1063
    PACKAGEINSETNR_IS_HIGHER_THEN_PACKAGEINSET = 1065
    MISSING_RETURN_CZ_SERVICE_RECIPIENT_ADDRESS = 1067
    MISSING_RETURN_CZ_BACKPACKNUMBER = 1068

    # CreatePickUpOrders
    BAD_SENDER_ADDRESS_200X = 2001
    BAD_COUNT_OF_PACKAGES = 2002
    INVALID_COUNTRY_IN_SENDER_ADDRESS_200X = 2005
    SENDDATE_MUST_BE_EQUAL_OR_GREATER_THAN_THE_CURRENT_DATE = 2006
    PARAMETER_SENDDATE_MUST_BE_FILLED = 2008

    # CreateOrders
    BAD_RECIPIENT_ADDRESS_300X = 3001
    BAD_SENDER_ADDRESS_300X = 3002
    BAD_COUNT_OF_PACKAGES_300X = 3003
    UNKNOWN_PRODUCT_TYPE_FOR_PACKAGE = 3004
    INVALID_COUNTRY_IN_SENDER_ADDRESS_300X = 3005
    INVALID_COUNTRY_IN_RECIPIENT_ADDRESS_300X = 3006
    SENDDATE_MUST_BE_EQUAL_OR_GREATER_THAN_THE_CURRENT_DATE_300X = 3007
    PARAMETER_SENDDATE_MUST_BE_FILLED_300X = 3008
    DUPLICIT_ORDREFID = 3010
    COUNT_OF_ORDERS_IS_GREATER_THAN_MAX_LIMIT = 3011
    LIMIT_OF_PACKAGES_PER_ORDER = 3012
    
    
    # GetNumberRange
    YOU_HAVE_ALREADY_UNSUSED_NUMBERS_OF_REQUESTED_RANGE_CONTACT_SALES_OR_TECHNICIAN_OR_IT = 4000
    PACKPRODUCTTYPE_DOES_NOT_EXIST = 4001
    QUANTITY_IS_ABOVE_MAXIMUM = 4002
    QUANTITY_IS_BELLOW_MINIMUM = 4003
    GLOBAL_ERROR_AS36 = 'AS36'

class Product(Enum2):
    PPL_PARCEL_CZ_BUSINESS = 'BUSS'
    PPL_PARCEL_CZ_BUSINESS_COD = 'BUSD'
    PPL_PARCEL_CZ_MORNING_PACKAGE = 'DOPO'
    PPL_PARCEL_CZ_MORNING_PACKAGE_COD = 'DOPD'
    EXPORT_PACKAGE = 'COPL'
    EXPORT_PACKAGE_COD = 'BUED'
    PPL_PARCEL_IMPORT = 'IMPO'
    PPL_PARCEL_CZ_PRIVATE = 'PRIV'
    PPL_PARCEL_CZ_PRIVATE_COD = 'PRID'
    PPL_PARCEL_CONNECT = 'CONN'
    PPL_PARCEL_CONNECT_COD = 'COND'
    PPL_PARCEL_CZ_RETURN = 'RETD'
    PPL_PARCEL_CZ_SMART = 'SMAR'
    PPL_PARCEL_CZ_SMART_COD = 'SMAD'
    PPL_PARCEL_SMART_EUROPE = 'SMEU'
    PPL_PARCEL_SMART_EUROPE_COD = 'SMED'
    PPL_PARCEL_RETURN_CONNECT_IMPORT = 'RECI'
    PPL_PARCEL_RETURN_CONNECT_EU = 'RECE'

CASH_ON_DELIVERY = [
    Product.PPL_PARCEL_CZ_BUSINESS_COD,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE,
    Product.EXPORT_PACKAGE_COD,
    Product.PPL_PARCEL_CZ_PRIVATE_COD,
    Product.PPL_PARCEL_CONNECT_COD,
    Product.PPL_PARCEL_CZ_SMART_COD,
    Product.PPL_PARCEL_SMART_EUROPE_COD,
]

DELIVERY_SATURDAY = [
    Product.PPL_PARCEL_CZ_BUSINESS,
    Product.PPL_PARCEL_CZ_BUSINESS_COD,
    Product.PPL_PARCEL_CZ_PRIVATE,
    Product.PPL_PARCEL_CZ_PRIVATE_COD,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE_COD,
    Product.EXPORT_PACKAGE,
    Product.EXPORT_PACKAGE_COD,
    Product.PPL_PARCEL_CZ_SMART,
    Product.PPL_PARCEL_CZ_SMART_COD,
    Product.PPL_PARCEL_SMART_EUROPE,
    Product.PPL_PARCEL_SMART_EUROPE_COD,
]

PRIVATE_PRODUCTS = [
    Product.PPL_PARCEL_CZ_PRIVATE,
    Product.PPL_PARCEL_CZ_PRIVATE_COD,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE_COD,
]

WEIGHTED_PRODUCTS = [
    Product.PPL_PARCEL_CZ_BUSINESS,
    Product.PPL_PARCEL_CZ_BUSINESS_COD,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE_COD,
    Product.EXPORT_PACKAGE,
    Product.EXPORT_PACKAGE_COD,
    Product.PPL_PARCEL_CZ_PRIVATE,
    Product.PPL_PARCEL_CZ_PRIVATE_COD,
    Product.PPL_PARCEL_CONNECT,
    Product.PPL_PARCEL_CONNECT_COD,
]

PARCEL_SHOP_PRODUCTS = [
    Product.PPL_PARCEL_CZ_PRIVATE,
    Product.PPL_PARCEL_CZ_PRIVATE_COD,
    Product.PPL_PARCEL_CONNECT,
    Product.PPL_PARCEL_CONNECT_COD,
    Product.PPL_PARCEL_CZ_SMART,
    Product.PPL_PARCEL_CZ_SMART_COD,
    Product.PPL_PARCEL_SMART_EUROPE,
    Product.PPL_PARCEL_SMART_EUROPE_COD,
]

DELIVERY_DOMESTIC = [
    Product.PPL_PARCEL_CZ_BUSINESS,
    Product.PPL_PARCEL_CZ_BUSINESS_COD,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE,
    Product.PPL_PARCEL_CZ_MORNING_PACKAGE_COD,
    Product.PPL_PARCEL_CZ_PRIVATE,
    Product.PPL_PARCEL_CZ_PRIVATE_COD,
    Product.PPL_PARCEL_CZ_RETURN,
    Product.PPL_PARCEL_CZ_SMART,
    Product.PPL_PARCEL_CZ_SMART_COD,

]

DELIVERY_INTERNATIONAL = [
    Product.PPL_PARCEL_CZ_BUSINESS,
    Product.PPL_PARCEL_CZ_BUSINESS_COD,
    Product.PPL_PARCEL_CONNECT,
    Product.PPL_PARCEL_CONNECT_COD,
    Product.PPL_PARCEL_SMART_EUROPE,
    Product.PPL_PARCEL_SMART_EUROPE_COD,
    Product.PPL_PARCEL_IMPORT,
    Product.PPL_PARCEL_RETURN_CONNECT_IMPORT,
    Product.PPL_PARCEL_RETURN_CONNECT_EU,
]

class PackageStatus(Enum2):
    DATA_SHIPMENT = 900
    PICKED_UP_FROM_SENDER = 150
    PICKED_UP_FROM_SENDER_WITH_CLAUSE = 151
    PICKED_UP_FROM_SENDER_EXPORT = 170
    DELIVERED = 450
    DELIVERED_UNCOMPLETE = 451
    DELIVERED_WITH_CLAUSE = 452
    NOT_DELIVERED = 336
    NOT_DELIVERED_RUN_OUT_OF_TIME = 454
    NOT_DELIVERED_LATE_DEPO_ARRIVAL = 455
    NOT_DELIVERED_CAR_ISSUE = 456
    NOT_DELIVERED_BAD_WEATHER = 457
    NOT_DELIVERED_RECIPIENT_NOT_ADDRESSED_NOTIFICATION = 458
    NOT_DELIVERED_RECIPIENT_NOT_ADDRESSED = 459
    NOT_DELIVERED_OTHER_TERM = 460
    NOT_DELIVERED_PERSONAL_PICKUP = 461
    NOT_DELIVERED_NEW_ADDRESS = 462
    NOT_DELIVERED_DID_NOT_ORDER = 463
    NOT_DELIVERED_SHIPPED_LATE = 464
    NOT_DELIVERED_PACKAGE_UNCOMPLETE = 465
    NOT_DELIVERED_OTHER_REASON = 466
    NOT_DELIVERED_ADDRESS_NOT_FOUND = 467
    NOT_DELIVERED_NO_CASH = 468
    NOT_DELIVERED_HOLIDAYS = 469
    NOT_DELIVERED_DETAINED_AT_CUSTOMS = 470
    NOT_DELIVERED_WRONG_DOCUMENTS = 471
    NOT_DELIVERED_DAMAGED = 472
    NOT_DELIVERED_CLOSED = 473
    NOT_DELIVERED_CANT_CONTACT_SENDER = 993
    BACK_TO_SENDER = 500
    OUT_FOR_DELIVERY = 281
    OUT_FOR_DELIVERY_AROAD = 282
    OUT_FOR_DELIVERY_AT_DEPO = 300
    OUT_FOR_DELIVERY_STORED_KTM = 343
    OUT_FOR_DELIVERY_RECORD_PL = 400
    OUT_FOR_DELIVERY_ACCEPT_AT_CENTER = 500
    OUT_FOR_DELIVERY_ISSUED_TO_PARTNER = 720
    OUT_FOR_DELIVERY_ACCEPTED_AT_EXPORT_DEPO = 733
    OUT_FOR_DELIVERY_ISSUED_FROM_EXPORT_DEPO = 734
    COD_PAID_TO_CUSTOMER_REQUEST = 605
    COD_PAID_TO_CUSTOMER = 606
    COD_PAID_TO_PARTNER_REQUEST = 613
    COD_PAID_TO_PARTNER = 614


class Flag(Enum2):
    CL = 'CL'
    SL = 'SL'
    SL2 = 'SL2'

class ExternalNumber(Enum2):
    B2CO = 'B2CO'
    CUST = 'CUST'
    VARS = 'VARS'
    PSN = 'PSN'
    PJJD = 'PJJD'

class Directions(Enum2):
    IN = 'IN'
    OUT = 'OUT'

class Services(Enum2):
    COD = 'COD' # Cash on delivery
    EXW = 'EXW' # cash only on delivery
    INSR = 'INSR' # insurance
    MD = 'MD' # mid day
    A15 = 'A15' # age 15
    A18 = 'A18' # age 18
    PUBC = 'PUBC' # pickup by driver
    DPOD = 'DPOD' # additional delivery attempt

LABEL_SERVICES = [
    # Services.A15, # apparently not supported but it's in docs..
    # Services.A18, # apparently not supported but it's in docs..
    Services.PUBC,
    Services.DPOD,
]

class Age(Enum2):
    A15 = 'A15'
    A18 = 'A18'

class AddressType(Enum2):
    BP = 'BP' # address to return undelivered packages
    RETD = 'RETD' # address to return return packages
    RETC = 'RETC' # address to return return connect packages


class LabelReturnChanel(Enum2):
    EMAIL = 'Email'
    FTP = 'Ftp'
    HTTP = 'Http'

class LabelSettingModel(Enum2):
    PDF = 'Pdf'
    ZPL = 'Zpl'
    PNG = 'Png'
    JPG = 'Jpg'
    SVG = 'Svg'


class ImportStatus(Enum2):
    ACCEPTED = 'Accepted'
    INPROCESS = 'InProcess'
    COMPLETE = 'Complete'
    ERROR = 'Error'